"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var anyworker_1 = __importDefault(require("anyworker"));
var eev_1 = __importDefault(require("eev"));
var default_1 = /** @class */ (function () {
    function default_1() {
        var _this = this;
        this.kill = function () { return _this._worker.terminate(); };
        this.task = function (func) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (typeof func !== 'function') {
                throw new TypeError('Passed parameter is not a function');
            }
            if (!_this.isRunning)
                throw new Error('This AsyncWorker was already killed and is no longer running');
            return new Promise(function (resolve, reject) {
                var msg = function (result) {
                    _this._eev.off('task_resolve', msg);
                    _this._eev.off('task_reject', err);
                    resolve(result);
                };
                var err = function (error) {
                    _this._eev.off('task_reject', err);
                    _this._eev.off('task_resolve', msg);
                    if (typeof error === 'string' && error.includes('DataCloneError')) {
                        reject(new TypeError('DataCloneError: Your task function returns a non-transferable value'));
                    }
                    else {
                        reject(error);
                    }
                };
                _this._eev.on('task_resolve', msg);
                _this._eev.on('task_reject', err);
                try {
                    _this._worker.postMessage({ __from: 'task', funcStr: func.toString(), args: args });
                }
                catch (error) {
                    _this._eev.emit('task_reject', error);
                }
            });
        };
        this.cook = function (func) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (typeof func !== 'function') {
                throw new TypeError('Passed parameter is not a function');
            }
            if (!_this.isRunning)
                throw new Error('This AsyncWorker was already killed and is no longer running');
            var funcId = '_' +
                Math.random()
                    .toString(36)
                    .substr(2, 9);
            var outerErr = function (error) {
                _this._eev.off('cook_reject_outer', outerErr);
                throw error;
            };
            _this._eev.on('cook_reject_outer', outerErr);
            try {
                _this._worker.postMessage({ __from: 'cook', funcStr: func.toString(), funcId: funcId, args: args });
            }
            catch (error) {
                _this._eev.emit('cook_reject_outer', error);
            }
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return new Promise(function (resolve, reject) {
                    var msg = function (result) {
                        _this._eev.off('cook_resolve', msg);
                        _this._eev.off('cook_reject', err);
                        resolve(result);
                    };
                    var err = function (error) {
                        _this._eev.off('cook_resolve', msg);
                        _this._eev.off('cook_reject', err);
                        if (typeof error !== 'string' && error.name === 'DataCloneError') {
                            reject(new TypeError('DataCloneError: Your cook function returns a non-transferable value'));
                        }
                        else {
                            reject(error);
                        }
                    };
                    _this._eev.on('cook_resolve', msg);
                    _this._eev.on('cook_reject', err);
                    try {
                        _this._worker.postMessage({ __from: 'cook', args: args, funcId: funcId });
                    }
                    catch (error) {
                        _this._eev.emit('cook_reject', error);
                    }
                });
            };
        };
        this.track = function (func) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (typeof func !== 'function') {
                throw new TypeError('Passed parameter is not a function');
            }
            if (!_this.isRunning)
                throw new Error('This AsyncWorker was already killed and is no longer running');
            var inTicker;
            var result = new Promise(function (resolve, reject) {
                var msg = function (result) {
                    if (result.progress !== undefined) {
                        if (inTicker)
                            inTicker(result.progress);
                    }
                    else {
                        _this._eev.off('track_resolve', msg);
                        _this._eev.off('track_reject', err);
                        resolve(result);
                    }
                };
                var err = function (error) {
                    _this._eev.off('track_resolve', msg);
                    _this._eev.off('track_reject', err);
                    if (typeof error === 'string' && error.includes('DataCloneError')) {
                        reject(new TypeError('DataCloneError: Your track function returns a non-transferable value'));
                    }
                    else {
                        reject(error);
                    }
                };
                _this._eev.on('track_resolve', msg);
                _this._eev.on('track_reject', err);
            });
            var tick = function (ticker) {
                inTicker = ticker;
            };
            try {
                _this._worker.postMessage({ __from: 'track', funcStr: func.toString(), args: args });
            }
            catch (error) {
                result.catch(function () { });
                throw error;
            }
            return { result: result, tick: tick };
        };
        this._worker = new anyworker_1.default("() => {\n\t\t\tconst slaveFuncs = {}\n\n\t\t\tonmessage(({ __from, funcStr, args, funcId }) => {\n\t\t\t\tswitch (__from) {\n\t\t\t\t\tcase 'task':\n\t\t\t\t\t\ttry {\n\t\t\t\t\t\t\tconst result = new Function('return (' + funcStr + ')')()(...args)\n\t\t\t\t\t\t\tpostMessage({ __from, result })\n\t\t\t\t\t\t} catch (error) {\n\t\t\t\t\t\t\tpostMessage({ __from, error })\n\t\t\t\t\t\t}\n\t\t\t\t\t\tbreak\n\n\t\t\t\t\tcase 'cook':\n\t\t\t\t\t\ttry {\n\t\t\t\t\t\t\tif (funcId && args && funcStr) {\n\t\t\t\t\t\t\t\tslaveFuncs[funcId] = new Function('return (' + funcStr + ')')()(...args)\n\t\t\t\t\t\t\t} else {\n\t\t\t\t\t\t\t\tconst result = slaveFuncs[funcId](...args)\n\t\t\t\t\t\t\t\tpostMessage({ __from, result })\n\t\t\t\t\t\t\t}\n\t\t\t\t\t\t} catch (error) {\n\t\t\t\t\t\t\tif(error.name === 'DataCloneError')\n\t\t\t\t\t\t\t\terror = { name: error.name, message: error.message }\n\t\t\t\t\t\t\tpostMessage({ __from, error })\n\t\t\t\t\t\t}\n\t\t\t\t\t\tbreak\n\n\t\t\t\t\tcase 'track':\n\t\t\t\t\t\ttry {\n\t\t\t\t\t\t\tconst tick = progress => postMessage({ __from, progress })\n\t\t\t\t\t\t\tconst result = new Function('return (' + funcStr + ')')()(tick, ...args)\n\t\t\t\t\t\t\tpostMessage({ __from, result })\n\t\t\t\t\t\t} catch (error) {\n\t\t\t\t\t\t\tpostMessage({ __from, error })\n\t\t\t\t\t\t}\n\t\t\t\t\t\tbreak\n\n\t\t\t\t\tdefault:\n\t\t\t\t\t\tbreak\n\t\t\t\t}\n\t\t\t})\n\t\t}");
        this._eev = new eev_1.default();
        this._worker.onmessage(function (_a) {
            var __from = _a.__from, result = _a.result, error = _a.error, progress = _a.progress;
            if (progress !== undefined)
                _this._eev.emit(__from + "_resolve", { progress: progress });
            else if (error)
                _this._eev.emit(__from + "_reject", error);
            else
                _this._eev.emit(__from + "_resolve", result);
        });
        this._worker.onerror(function (error) {
            _this._worker.restart();
            ['task', 'cook', 'track'].forEach(function (name) { return _this._eev.emit(name + "_reject", error); });
        });
    }
    Object.defineProperty(default_1.prototype, "isRunning", {
        get: function () {
            return this._worker.isRunning;
        },
        enumerable: true,
        configurable: true
    });
    return default_1;
}());
exports.default = default_1;
